# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    MediaListResponse,
    MediaUpdateResponse,
    MediaUploadResponse,
    MediaRetrieveResponse,
)
from telnyx._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMedia:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        media = client.media.retrieve(
            "media_name",
        )
        assert_matches_type(MediaRetrieveResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.media.with_raw_response.retrieve(
            "media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = response.parse()
        assert_matches_type(MediaRetrieveResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.media.with_streaming_response.retrieve(
            "media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = response.parse()
            assert_matches_type(MediaRetrieveResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            client.media.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        media = client.media.update(
            media_name="media_name",
        )
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        media = client.media.update(
            media_name="media_name",
            media_url="http://www.example.com/audio.mp3",
            ttl_secs=86400,
        )
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.media.with_raw_response.update(
            media_name="media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = response.parse()
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.media.with_streaming_response.update(
            media_name="media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = response.parse()
            assert_matches_type(MediaUpdateResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            client.media.with_raw_response.update(
                media_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        media = client.media.list()
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        media = client.media.list(
            filter={"content_type": ["application_xml"]},
        )
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.media.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = response.parse()
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.media.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = response.parse()
            assert_matches_type(MediaListResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        media = client.media.delete(
            "media_name",
        )
        assert media is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.media.with_raw_response.delete(
            "media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = response.parse()
        assert media is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.media.with_streaming_response.delete(
            "media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = response.parse()
            assert media is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            client.media.with_raw_response.delete(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_download(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        media = client.media.download(
            "media_name",
        )
        assert media.is_closed
        assert media.json() == {"foo": "bar"}
        assert cast(Any, media.is_closed) is True
        assert isinstance(media, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_download(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        media = client.media.with_raw_response.download(
            "media_name",
        )

        assert media.is_closed is True
        assert media.http_request.headers.get("X-Stainless-Lang") == "python"
        assert media.json() == {"foo": "bar"}
        assert isinstance(media, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_download(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.media.with_streaming_response.download(
            "media_name",
        ) as media:
            assert not media.is_closed
            assert media.http_request.headers.get("X-Stainless-Lang") == "python"

            assert media.json() == {"foo": "bar"}
            assert cast(Any, media.is_closed) is True
            assert isinstance(media, StreamedBinaryAPIResponse)

        assert cast(Any, media.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_download(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            client.media.with_raw_response.download(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_upload(self, client: Telnyx) -> None:
        media = client.media.upload(
            media_url="http://www.example.com/audio.mp3",
        )
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_upload_with_all_params(self, client: Telnyx) -> None:
        media = client.media.upload(
            media_url="http://www.example.com/audio.mp3",
            media_name="my-file",
            ttl_secs=86400,
        )
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_upload(self, client: Telnyx) -> None:
        response = client.media.with_raw_response.upload(
            media_url="http://www.example.com/audio.mp3",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = response.parse()
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_upload(self, client: Telnyx) -> None:
        with client.media.with_streaming_response.upload(
            media_url="http://www.example.com/audio.mp3",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = response.parse()
            assert_matches_type(MediaUploadResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMedia:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.retrieve(
            "media_name",
        )
        assert_matches_type(MediaRetrieveResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.media.with_raw_response.retrieve(
            "media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = await response.parse()
        assert_matches_type(MediaRetrieveResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.media.with_streaming_response.retrieve(
            "media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = await response.parse()
            assert_matches_type(MediaRetrieveResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            await async_client.media.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.update(
            media_name="media_name",
        )
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.update(
            media_name="media_name",
            media_url="http://www.example.com/audio.mp3",
            ttl_secs=86400,
        )
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.media.with_raw_response.update(
            media_name="media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = await response.parse()
        assert_matches_type(MediaUpdateResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.media.with_streaming_response.update(
            media_name="media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = await response.parse()
            assert_matches_type(MediaUpdateResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            await async_client.media.with_raw_response.update(
                media_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.list()
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.list(
            filter={"content_type": ["application_xml"]},
        )
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.media.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = await response.parse()
        assert_matches_type(MediaListResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.media.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = await response.parse()
            assert_matches_type(MediaListResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.delete(
            "media_name",
        )
        assert media is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.media.with_raw_response.delete(
            "media_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = await response.parse()
        assert media is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.media.with_streaming_response.delete(
            "media_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = await response.parse()
            assert media is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            await async_client.media.with_raw_response.delete(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_download(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        media = await async_client.media.download(
            "media_name",
        )
        assert media.is_closed
        assert await media.json() == {"foo": "bar"}
        assert cast(Any, media.is_closed) is True
        assert isinstance(media, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_download(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        media = await async_client.media.with_raw_response.download(
            "media_name",
        )

        assert media.is_closed is True
        assert media.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await media.json() == {"foo": "bar"}
        assert isinstance(media, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_download(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/media/media_name/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.media.with_streaming_response.download(
            "media_name",
        ) as media:
            assert not media.is_closed
            assert media.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await media.json() == {"foo": "bar"}
            assert cast(Any, media.is_closed) is True
            assert isinstance(media, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, media.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_download(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `media_name` but received ''"):
            await async_client.media.with_raw_response.download(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_upload(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.upload(
            media_url="http://www.example.com/audio.mp3",
        )
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_upload_with_all_params(self, async_client: AsyncTelnyx) -> None:
        media = await async_client.media.upload(
            media_url="http://www.example.com/audio.mp3",
            media_name="my-file",
            ttl_secs=86400,
        )
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_upload(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.media.with_raw_response.upload(
            media_url="http://www.example.com/audio.mp3",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        media = await response.parse()
        assert_matches_type(MediaUploadResponse, media, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_upload(self, async_client: AsyncTelnyx) -> None:
        async with async_client.media.with_streaming_response.upload(
            media_url="http://www.example.com/audio.mp3",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            media = await response.parse()
            assert_matches_type(MediaUploadResponse, media, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PortingOrderActivationSettings"]


class PortingOrderActivationSettings(BaseModel):
    activation_status: Optional[
        Literal[
            "New",
            "Pending",
            "Conflict",
            "Cancel Pending",
            "Failed",
            "Concurred",
            "Activate RDY",
            "Disconnect Pending",
            "Concurrence Sent",
            "Old",
            "Sending",
            "Active",
            "Cancelled",
        ]
    ] = None
    """Activation status"""

    fast_port_eligible: Optional[bool] = None
    """Indicates whether this porting order is eligible for FastPort"""

    foc_datetime_actual: Optional[datetime] = None
    """ISO 8601 formatted Date/Time of the FOC date"""

    foc_datetime_requested: Optional[datetime] = None
    """ISO 8601 formatted Date/Time requested for the FOC date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionChangeBundleStatusParams"]


class ActionChangeBundleStatusParams(TypedDict, total=False):
    bundle_id: Required[str]
    """The new bundle_id setting for the number.

    If you are assigning the number to a bundle, this is the unique ID of the bundle
    you wish to use. If you are removing the number from a bundle, this must be
    null. You cannot assign a number from one bundle to another directly. You must
    first remove it from a bundle, and then assign it to a new bundle.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .usage_reports.usage_reports import (
    UsageReportsResource,
    AsyncUsageReportsResource,
    UsageReportsResourceWithRawResponse,
    AsyncUsageReportsResourceWithRawResponse,
    UsageReportsResourceWithStreamingResponse,
    AsyncUsageReportsResourceWithStreamingResponse,
)
from .batch_detail_records.batch_detail_records import (
    BatchDetailRecordsResource,
    AsyncBatchDetailRecordsResource,
    BatchDetailRecordsResourceWithRawResponse,
    AsyncBatchDetailRecordsResourceWithRawResponse,
    BatchDetailRecordsResourceWithStreamingResponse,
    AsyncBatchDetailRecordsResourceWithStreamingResponse,
)

__all__ = ["ReportingResource", "AsyncReportingResource"]


class ReportingResource(SyncAPIResource):
    @cached_property
    def batch_detail_records(self) -> BatchDetailRecordsResource:
        return BatchDetailRecordsResource(self._client)

    @cached_property
    def usage_reports(self) -> UsageReportsResource:
        return UsageReportsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ReportingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ReportingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ReportingResourceWithStreamingResponse(self)


class AsyncReportingResource(AsyncAPIResource):
    @cached_property
    def batch_detail_records(self) -> AsyncBatchDetailRecordsResource:
        return AsyncBatchDetailRecordsResource(self._client)

    @cached_property
    def usage_reports(self) -> AsyncUsageReportsResource:
        return AsyncUsageReportsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncReportingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncReportingResourceWithStreamingResponse(self)


class ReportingResourceWithRawResponse:
    def __init__(self, reporting: ReportingResource) -> None:
        self._reporting = reporting

    @cached_property
    def batch_detail_records(self) -> BatchDetailRecordsResourceWithRawResponse:
        return BatchDetailRecordsResourceWithRawResponse(self._reporting.batch_detail_records)

    @cached_property
    def usage_reports(self) -> UsageReportsResourceWithRawResponse:
        return UsageReportsResourceWithRawResponse(self._reporting.usage_reports)


class AsyncReportingResourceWithRawResponse:
    def __init__(self, reporting: AsyncReportingResource) -> None:
        self._reporting = reporting

    @cached_property
    def batch_detail_records(self) -> AsyncBatchDetailRecordsResourceWithRawResponse:
        return AsyncBatchDetailRecordsResourceWithRawResponse(self._reporting.batch_detail_records)

    @cached_property
    def usage_reports(self) -> AsyncUsageReportsResourceWithRawResponse:
        return AsyncUsageReportsResourceWithRawResponse(self._reporting.usage_reports)


class ReportingResourceWithStreamingResponse:
    def __init__(self, reporting: ReportingResource) -> None:
        self._reporting = reporting

    @cached_property
    def batch_detail_records(self) -> BatchDetailRecordsResourceWithStreamingResponse:
        return BatchDetailRecordsResourceWithStreamingResponse(self._reporting.batch_detail_records)

    @cached_property
    def usage_reports(self) -> UsageReportsResourceWithStreamingResponse:
        return UsageReportsResourceWithStreamingResponse(self._reporting.usage_reports)


class AsyncReportingResourceWithStreamingResponse:
    def __init__(self, reporting: AsyncReportingResource) -> None:
        self._reporting = reporting

    @cached_property
    def batch_detail_records(self) -> AsyncBatchDetailRecordsResourceWithStreamingResponse:
        return AsyncBatchDetailRecordsResourceWithStreamingResponse(self._reporting.batch_detail_records)

    @cached_property
    def usage_reports(self) -> AsyncUsageReportsResourceWithStreamingResponse:
        return AsyncUsageReportsResourceWithStreamingResponse(self._reporting.usage_reports)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .autoresp_configs import (
    AutorespConfigsResource,
    AsyncAutorespConfigsResource,
    AutorespConfigsResourceWithRawResponse,
    AsyncAutorespConfigsResourceWithRawResponse,
    AutorespConfigsResourceWithStreamingResponse,
    AsyncAutorespConfigsResourceWithStreamingResponse,
)
from .messaging_profiles import (
    MessagingProfilesResource,
    AsyncMessagingProfilesResource,
    MessagingProfilesResourceWithRawResponse,
    AsyncMessagingProfilesResourceWithRawResponse,
    MessagingProfilesResourceWithStreamingResponse,
    AsyncMessagingProfilesResourceWithStreamingResponse,
)

__all__ = [
    "AutorespConfigsResource",
    "AsyncAutorespConfigsResource",
    "AutorespConfigsResourceWithRawResponse",
    "AsyncAutorespConfigsResourceWithRawResponse",
    "AutorespConfigsResourceWithStreamingResponse",
    "AsyncAutorespConfigsResourceWithStreamingResponse",
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "MessagingProfilesResource",
    "AsyncMessagingProfilesResource",
    "MessagingProfilesResourceWithRawResponse",
    "AsyncMessagingProfilesResourceWithRawResponse",
    "MessagingProfilesResourceWithStreamingResponse",
    "AsyncMessagingProfilesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import texml_secrets_params
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .accounts.accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from ...types.texml_secrets_response import TexmlSecretsResponse

__all__ = ["TexmlResource", "AsyncTexmlResource"]


class TexmlResource(SyncAPIResource):
    @cached_property
    def accounts(self) -> AccountsResource:
        return AccountsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TexmlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TexmlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TexmlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TexmlResourceWithStreamingResponse(self)

    def secrets(
        self,
        *,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TexmlSecretsResponse:
        """
        Create a TeXML secret which can be later used as a Dynamic Parameter for TeXML
        when using Mustache Templates in your TeXML. In your TeXML you will be able to
        use your secret name, and this name will be replaced by the actual secret value
        when processing the TeXML on Telnyx side. The secrets are not visible in any
        logs.

        Args:
          name: Name used as a reference for the secret, if the name already exists within the
              account its value will be replaced

          value: Secret value which will be used when rendering the TeXML template

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/texml/secrets",
            body=maybe_transform(
                {
                    "name": name,
                    "value": value,
                },
                texml_secrets_params.TexmlSecretsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TexmlSecretsResponse,
        )


class AsyncTexmlResource(AsyncAPIResource):
    @cached_property
    def accounts(self) -> AsyncAccountsResource:
        return AsyncAccountsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTexmlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTexmlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTexmlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTexmlResourceWithStreamingResponse(self)

    async def secrets(
        self,
        *,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TexmlSecretsResponse:
        """
        Create a TeXML secret which can be later used as a Dynamic Parameter for TeXML
        when using Mustache Templates in your TeXML. In your TeXML you will be able to
        use your secret name, and this name will be replaced by the actual secret value
        when processing the TeXML on Telnyx side. The secrets are not visible in any
        logs.

        Args:
          name: Name used as a reference for the secret, if the name already exists within the
              account its value will be replaced

          value: Secret value which will be used when rendering the TeXML template

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/texml/secrets",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "value": value,
                },
                texml_secrets_params.TexmlSecretsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TexmlSecretsResponse,
        )


class TexmlResourceWithRawResponse:
    def __init__(self, texml: TexmlResource) -> None:
        self._texml = texml

        self.secrets = to_raw_response_wrapper(
            texml.secrets,
        )

    @cached_property
    def accounts(self) -> AccountsResourceWithRawResponse:
        return AccountsResourceWithRawResponse(self._texml.accounts)


class AsyncTexmlResourceWithRawResponse:
    def __init__(self, texml: AsyncTexmlResource) -> None:
        self._texml = texml

        self.secrets = async_to_raw_response_wrapper(
            texml.secrets,
        )

    @cached_property
    def accounts(self) -> AsyncAccountsResourceWithRawResponse:
        return AsyncAccountsResourceWithRawResponse(self._texml.accounts)


class TexmlResourceWithStreamingResponse:
    def __init__(self, texml: TexmlResource) -> None:
        self._texml = texml

        self.secrets = to_streamed_response_wrapper(
            texml.secrets,
        )

    @cached_property
    def accounts(self) -> AccountsResourceWithStreamingResponse:
        return AccountsResourceWithStreamingResponse(self._texml.accounts)


class AsyncTexmlResourceWithStreamingResponse:
    def __init__(self, texml: AsyncTexmlResource) -> None:
        self._texml = texml

        self.secrets = async_to_streamed_response_wrapper(
            texml.secrets,
        )

    @cached_property
    def accounts(self) -> AsyncAccountsResourceWithStreamingResponse:
        return AsyncAccountsResourceWithStreamingResponse(self._texml.accounts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CsvDownload"]


class CsvDownload(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    status: Optional[Literal["pending", "complete", "failed", "expired"]] = None
    """Indicates the completion level of the CSV report.

    Only complete CSV download requests will be able to be retrieved.
    """

    url: Optional[str] = None
    """The URL at which the CSV file can be retrieved."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["NumberLookupRetrieveResponse", "Data", "DataCallerName", "DataCarrier", "DataPortability"]


class DataCallerName(BaseModel):
    caller_name: Optional[str] = None
    """The name of the requested phone number's owner as per the CNAM database"""

    error_code: Optional[str] = None
    """
    A caller-name lookup specific error code, expressed as a stringified 5-digit
    integer
    """


class DataCarrier(BaseModel):
    error_code: Optional[str] = None
    """Unused"""

    mobile_country_code: Optional[str] = None
    """
    Region code that matches the specific country calling code if the requested
    phone number type is mobile
    """

    mobile_network_code: Optional[str] = None
    """
    National destination code (NDC), with a 0 prefix, if an NDC is found and the
    requested phone number type is mobile
    """

    name: Optional[str] = None
    """
    SPID (Service Provider ID) name, if the requested phone number has been ported;
    otherwise, the name of carrier who owns the phone number block
    """

    normalized_carrier: Optional[str] = None
    """If known to Telnyx and applicable, the primary network carrier."""

    type: Optional[
        Literal[
            "fixed line",
            "mobile",
            "voip",
            "fixed line or mobile",
            "toll free",
            "premium rate",
            "shared cost",
            "personal number",
            "pager",
            "uan",
            "voicemail",
            "unknown",
        ]
    ] = None
    """
    A phone number type that identifies the type of service associated with the
    requested phone number
    """


class DataPortability(BaseModel):
    altspid: Optional[str] = None
    """Alternative SPID (Service Provider ID).

    Often used when a carrier is using a number from another carrier
    """

    altspid_carrier_name: Optional[str] = None
    """Alternative service provider name"""

    altspid_carrier_type: Optional[str] = None
    """Alternative service provider type"""

    city: Optional[str] = None
    """
    City name extracted from the locality in the Local Exchange Routing Guide (LERG)
    database
    """

    line_type: Optional[str] = None
    """Type of number"""

    lrn: Optional[str] = None
    """Local Routing Number, if assigned to the requested phone number"""

    ocn: Optional[str] = None
    """
    Operating Company Name (OCN) as per the Local Exchange Routing Guide (LERG)
    database
    """

    ported_date: Optional[str] = None
    """ISO-formatted date when the requested phone number has been ported"""

    ported_status: Optional[Literal["Y", "N", ""]] = None
    """Indicates whether or not the requested phone number has been ported"""

    spid: Optional[str] = None
    """SPID (Service Provider ID)"""

    spid_carrier_name: Optional[str] = None
    """Service provider name"""

    spid_carrier_type: Optional[str] = None
    """Service provider type"""

    state: Optional[str] = None


class Data(BaseModel):
    caller_name: Optional[DataCallerName] = None

    carrier: Optional[DataCarrier] = None

    country_code: Optional[str] = None
    """Region code that matches the specific country calling code"""

    fraud: Optional[str] = None
    """Unused"""

    national_format: Optional[str] = None
    """
    Hyphen-separated national number, preceded by the national destination code
    (NDC), with a 0 prefix, if an NDC is found
    """

    phone_number: Optional[str] = None
    """E164-formatted phone number"""

    portability: Optional[DataPortability] = None

    record_type: Optional[str] = None
    """Identifies the type of record"""


class NumberLookupRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ActionEndResponse", "Data"]


class Data(BaseModel):
    result: Optional[str] = None


class ActionEndResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["OAuthGrantsResponse"]


class OAuthGrantsResponse(BaseModel):
    redirect_uri: str
    """Redirect URI with authorization code or error"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["UploadCreateParams"]


class UploadCreateParams(TypedDict, total=False):
    number_ids: Required[SequenceNotStr[str]]

    additional_usages: List[Literal["calling_user_assignment", "first_party_app_assignment"]]

    civic_address_id: str
    """Identifies the civic address to assign all phone numbers to."""

    location_id: str
    """Identifies the location to assign all phone numbers to."""

    usage: Literal["calling_user_assignment", "first_party_app_assignment"]
    """The use case of the upload request.

    NOTE: `calling_user_assignment` is not supported for toll free numbers.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["OutboundCallRecordingParam"]


class OutboundCallRecordingParam(TypedDict, total=False):
    call_recording_caller_phone_numbers: SequenceNotStr[str]
    """
    When call_recording_type is 'by_caller_phone_number', only outbound calls using
    one of these numbers will be recorded. Numbers must be specified in E164 format.
    """

    call_recording_channels: Literal["single", "dual"]
    """
    When using 'dual' channels, the final audio file will be a stereo recording with
    the first leg on channel A, and the rest on channel B.
    """

    call_recording_format: Literal["wav", "mp3"]
    """The audio file format for calls being recorded."""

    call_recording_type: Literal["all", "none", "by_caller_phone_number"]
    """Specifies which calls are recorded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OAuthTokenParams"]


class OAuthTokenParams(TypedDict, total=False):
    grant_type: Required[Literal["client_credentials", "authorization_code", "refresh_token"]]
    """OAuth 2.0 grant type"""

    client_id: str
    """OAuth client ID (if not using HTTP Basic auth)"""

    client_secret: str
    """OAuth client secret (if not using HTTP Basic auth)"""

    code: str
    """Authorization code (for authorization_code flow)"""

    code_verifier: str
    """PKCE code verifier (for authorization_code flow)"""

    redirect_uri: str
    """Redirect URI (for authorization_code flow)"""

    refresh_token: str
    """Refresh token (for refresh_token flow)"""

    scope: str
    """Space-separated list of requested scopes (for client_credentials)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RoomRetrieveParams"]


class RoomRetrieveParams(TypedDict, total=False):
    include_sessions: bool
    """To decide if room sessions should be included in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ActionListParams"]


class ActionListParams(TypedDict, total=False):
    filter_sim_card_group_id: Annotated[str, PropertyInfo(alias="filter[sim_card_group_id]")]
    """A valid SIM card group ID."""

    filter_status: Annotated[Literal["in-progress", "completed", "failed"], PropertyInfo(alias="filter[status]")]
    """Filter by a specific status of the resource's lifecycle."""

    filter_type: Annotated[
        Literal[
            "set_private_wireless_gateway",
            "remove_private_wireless_gateway",
            "set_wireless_blocklist",
            "remove_wireless_blocklist",
        ],
        PropertyInfo(alias="filter[type]"),
    ]
    """Filter by action type."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load."""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = [
    "GlobalIPAssignmentsUsageRetrieveResponse",
    "Data",
    "DataGlobalIP",
    "DataGlobalIPAssignment",
    "DataGlobalIPAssignmentWireguardPeer",
    "DataReceived",
    "DataTransmitted",
]


class DataGlobalIP(BaseModel):
    id: Optional[str] = None
    """Global IP ID."""

    ip_address: Optional[str] = None
    """The Global IP address."""


class DataGlobalIPAssignmentWireguardPeer(BaseModel):
    ip_address: Optional[str] = None
    """The IP address of the interface."""

    name: Optional[str] = None
    """A user specified name for the interface."""


class DataGlobalIPAssignment(BaseModel):
    id: Optional[str] = None
    """Global IP assignment ID."""

    wireguard_peer: Optional[DataGlobalIPAssignmentWireguardPeer] = None

    wireguard_peer_id: Optional[str] = None
    """Wireguard peer ID."""


class DataReceived(BaseModel):
    amount: Optional[float] = None
    """The amount of data received."""

    unit: Optional[str] = None
    """The unit of the amount of data received."""


class DataTransmitted(BaseModel):
    amount: Optional[float] = None
    """The amount of data transmitted."""

    unit: Optional[str] = None
    """The unit of the amount of data transmitted."""


class Data(BaseModel):
    global_ip: Optional[DataGlobalIP] = None

    global_ip_assignment: Optional[DataGlobalIPAssignment] = None

    received: Optional[DataReceived] = None

    timestamp: Optional[datetime] = None
    """The timestamp of the metric."""

    transmitted: Optional[DataTransmitted] = None


class GlobalIPAssignmentsUsageRetrieveResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BrandVerifySMSOtpParams"]


class BrandVerifySMSOtpParams(TypedDict, total=False):
    otp_pin: Required[Annotated[str, PropertyInfo(alias="otpPin")]]
    """The OTP PIN received via SMS"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DialogflowConfigParam"]


class DialogflowConfigParam(TypedDict, total=False):
    analyze_sentiment: bool
    """Enable sentiment analysis from Dialogflow."""

    partial_automated_agent_reply: bool
    """Enable partial automated agent reply from Dialogflow."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .doc_service_document import DocServiceDocument

__all__ = ["DocumentUploadResponse"]


class DocumentUploadResponse(BaseModel):
    data: Optional[DocServiceDocument] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    notification_channel_list_params,
    notification_channel_create_params,
    notification_channel_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.notification_channel import NotificationChannel
from ..types.notification_channel_create_response import NotificationChannelCreateResponse
from ..types.notification_channel_delete_response import NotificationChannelDeleteResponse
from ..types.notification_channel_update_response import NotificationChannelUpdateResponse
from ..types.notification_channel_retrieve_response import NotificationChannelRetrieveResponse

__all__ = ["NotificationChannelsResource", "AsyncNotificationChannelsResource"]


class NotificationChannelsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NotificationChannelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NotificationChannelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NotificationChannelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NotificationChannelsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        channel_destination: str | Omit = omit,
        channel_type_id: Literal["sms", "voice", "email", "webhook"] | Omit = omit,
        notification_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelCreateResponse:
        """
        Create a notification channel.

        Args:
          channel_destination: The destination associated with the channel type.

          channel_type_id: A Channel Type ID

          notification_profile_id: A UUID reference to the associated Notification Profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/notification_channels",
            body=maybe_transform(
                {
                    "channel_destination": channel_destination,
                    "channel_type_id": channel_type_id,
                    "notification_profile_id": notification_profile_id,
                },
                notification_channel_create_params.NotificationChannelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelRetrieveResponse:
        """
        Get a notification channel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/notification_channels/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelRetrieveResponse,
        )

    def update(
        self,
        notification_channel_id: str,
        *,
        channel_destination: str | Omit = omit,
        channel_type_id: Literal["sms", "voice", "email", "webhook"] | Omit = omit,
        notification_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelUpdateResponse:
        """
        Update a notification channel.

        Args:
          channel_destination: The destination associated with the channel type.

          channel_type_id: A Channel Type ID

          notification_profile_id: A UUID reference to the associated Notification Profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not notification_channel_id:
            raise ValueError(
                f"Expected a non-empty value for `notification_channel_id` but received {notification_channel_id!r}"
            )
        return self._patch(
            f"/notification_channels/{notification_channel_id}",
            body=maybe_transform(
                {
                    "channel_destination": channel_destination,
                    "channel_type_id": channel_type_id,
                    "notification_profile_id": notification_profile_id,
                },
                notification_channel_update_params.NotificationChannelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelUpdateResponse,
        )

    def list(
        self,
        *,
        filter: notification_channel_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[NotificationChannel]:
        """
        List notification channels.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[associated_record_type][eq], filter[channel_type_id][eq],
              filter[notification_profile_id][eq], filter[notification_channel][eq],
              filter[notification_event_condition_id][eq], filter[status][eq]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/notification_channels",
            page=SyncDefaultFlatPagination[NotificationChannel],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    notification_channel_list_params.NotificationChannelListParams,
                ),
            ),
            model=NotificationChannel,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelDeleteResponse:
        """
        Delete a notification channel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/notification_channels/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelDeleteResponse,
        )


class AsyncNotificationChannelsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNotificationChannelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNotificationChannelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNotificationChannelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNotificationChannelsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        channel_destination: str | Omit = omit,
        channel_type_id: Literal["sms", "voice", "email", "webhook"] | Omit = omit,
        notification_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelCreateResponse:
        """
        Create a notification channel.

        Args:
          channel_destination: The destination associated with the channel type.

          channel_type_id: A Channel Type ID

          notification_profile_id: A UUID reference to the associated Notification Profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/notification_channels",
            body=await async_maybe_transform(
                {
                    "channel_destination": channel_destination,
                    "channel_type_id": channel_type_id,
                    "notification_profile_id": notification_profile_id,
                },
                notification_channel_create_params.NotificationChannelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelRetrieveResponse:
        """
        Get a notification channel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/notification_channels/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelRetrieveResponse,
        )

    async def update(
        self,
        notification_channel_id: str,
        *,
        channel_destination: str | Omit = omit,
        channel_type_id: Literal["sms", "voice", "email", "webhook"] | Omit = omit,
        notification_profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelUpdateResponse:
        """
        Update a notification channel.

        Args:
          channel_destination: The destination associated with the channel type.

          channel_type_id: A Channel Type ID

          notification_profile_id: A UUID reference to the associated Notification Profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not notification_channel_id:
            raise ValueError(
                f"Expected a non-empty value for `notification_channel_id` but received {notification_channel_id!r}"
            )
        return await self._patch(
            f"/notification_channels/{notification_channel_id}",
            body=await async_maybe_transform(
                {
                    "channel_destination": channel_destination,
                    "channel_type_id": channel_type_id,
                    "notification_profile_id": notification_profile_id,
                },
                notification_channel_update_params.NotificationChannelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelUpdateResponse,
        )

    def list(
        self,
        *,
        filter: notification_channel_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[NotificationChannel, AsyncDefaultFlatPagination[NotificationChannel]]:
        """
        List notification channels.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[associated_record_type][eq], filter[channel_type_id][eq],
              filter[notification_profile_id][eq], filter[notification_channel][eq],
              filter[notification_event_condition_id][eq], filter[status][eq]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/notification_channels",
            page=AsyncDefaultFlatPagination[NotificationChannel],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    notification_channel_list_params.NotificationChannelListParams,
                ),
            ),
            model=NotificationChannel,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NotificationChannelDeleteResponse:
        """
        Delete a notification channel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/notification_channels/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationChannelDeleteResponse,
        )


class NotificationChannelsResourceWithRawResponse:
    def __init__(self, notification_channels: NotificationChannelsResource) -> None:
        self._notification_channels = notification_channels

        self.create = to_raw_response_wrapper(
            notification_channels.create,
        )
        self.retrieve = to_raw_response_wrapper(
            notification_channels.retrieve,
        )
        self.update = to_raw_response_wrapper(
            notification_channels.update,
        )
        self.list = to_raw_response_wrapper(
            notification_channels.list,
        )
        self.delete = to_raw_response_wrapper(
            notification_channels.delete,
        )


class AsyncNotificationChannelsResourceWithRawResponse:
    def __init__(self, notification_channels: AsyncNotificationChannelsResource) -> None:
        self._notification_channels = notification_channels

        self.create = async_to_raw_response_wrapper(
            notification_channels.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            notification_channels.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            notification_channels.update,
        )
        self.list = async_to_raw_response_wrapper(
            notification_channels.list,
        )
        self.delete = async_to_raw_response_wrapper(
            notification_channels.delete,
        )


class NotificationChannelsResourceWithStreamingResponse:
    def __init__(self, notification_channels: NotificationChannelsResource) -> None:
        self._notification_channels = notification_channels

        self.create = to_streamed_response_wrapper(
            notification_channels.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            notification_channels.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            notification_channels.update,
        )
        self.list = to_streamed_response_wrapper(
            notification_channels.list,
        )
        self.delete = to_streamed_response_wrapper(
            notification_channels.delete,
        )


class AsyncNotificationChannelsResourceWithStreamingResponse:
    def __init__(self, notification_channels: AsyncNotificationChannelsResource) -> None:
        self._notification_channels = notification_channels

        self.create = async_to_streamed_response_wrapper(
            notification_channels.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            notification_channels.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            notification_channels.update,
        )
        self.list = async_to_streamed_response_wrapper(
            notification_channels.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            notification_channels.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .porting_report import PortingReport

__all__ = ["ReportCreateResponse"]


class ReportCreateResponse(BaseModel):
    data: Optional[PortingReport] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import date

from .._models import BaseModel

__all__ = ["InvoiceRetrieveResponse", "Data"]


class Data(BaseModel):
    download_url: Optional[str] = None
    """Present only if the query parameter `action=link` is set."""

    file_id: Optional[str] = None

    invoice_id: Optional[str] = None

    paid: Optional[bool] = None

    period_end: Optional[date] = None

    period_start: Optional[date] = None

    url: Optional[str] = None


class InvoiceRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .porting import (
    PortingResource,
    AsyncPortingResource,
    PortingResourceWithRawResponse,
    AsyncPortingResourceWithRawResponse,
    PortingResourceWithStreamingResponse,
    AsyncPortingResourceWithStreamingResponse,
)
from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from .loa_configurations import (
    LoaConfigurationsResource,
    AsyncLoaConfigurationsResource,
    LoaConfigurationsResourceWithRawResponse,
    AsyncLoaConfigurationsResourceWithRawResponse,
    LoaConfigurationsResourceWithStreamingResponse,
    AsyncLoaConfigurationsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "ReportsResource",
    "AsyncReportsResource",
    "ReportsResourceWithRawResponse",
    "AsyncReportsResourceWithRawResponse",
    "ReportsResourceWithStreamingResponse",
    "AsyncReportsResourceWithStreamingResponse",
    "LoaConfigurationsResource",
    "AsyncLoaConfigurationsResource",
    "LoaConfigurationsResourceWithRawResponse",
    "AsyncLoaConfigurationsResourceWithRawResponse",
    "LoaConfigurationsResourceWithStreamingResponse",
    "AsyncLoaConfigurationsResourceWithStreamingResponse",
    "PortingResource",
    "AsyncPortingResource",
    "PortingResourceWithRawResponse",
    "AsyncPortingResourceWithRawResponse",
    "PortingResourceWithStreamingResponse",
    "AsyncPortingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CallAIGatherEnded", "Payload", "PayloadMessageHistory"]


class PayloadMessageHistory(BaseModel):
    content: Optional[str] = None
    """The content of the message"""

    role: Optional[Literal["assistant", "user"]] = None
    """The role of the message sender"""


class Payload(BaseModel):
    call_control_id: Optional[str] = None
    """Call ID used to issue commands via Call Control API."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call and can be used to correlate webhook events."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    client_state: Optional[str] = None
    """State received from a command."""

    connection_id: Optional[str] = None
    """Telnyx connection ID used in the call."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """Number or SIP URI placing the call."""

    message_history: Optional[List[PayloadMessageHistory]] = None
    """The history of the messages exchanged during the AI gather"""

    result: Optional[Dict[str, object]] = None
    """
    The result of the AI gather, its type depends of the `parameters` provided in
    the command
    """

    status: Optional[Literal["valid", "invalid"]] = None
    """Reflects how command ended."""

    to: Optional[str] = None
    """Destination number or SIP URI of the call."""


class CallAIGatherEnded(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["call.ai_gather.ended"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[Payload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["RunCreateParams"]


class RunCreateParams(TypedDict, total=False):
    input: Dict[str, object]

    metadata: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.external_connections import (
    PhoneNumberUpdateResponse,
    PhoneNumberRetrieveResponse,
    ExternalConnectionPhoneNumber,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        phone_number = client.external_connections.phone_numbers.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.external_connections.phone_numbers.with_raw_response.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.external_connections.phone_numbers.with_streaming_response.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.phone_numbers.with_raw_response.retrieve(
                phone_number_id="1234567889",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.external_connections.phone_numbers.with_raw_response.retrieve(
                phone_number_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        phone_number = client.external_connections.phone_numbers.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.external_connections.phone_numbers.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.external_connections.phone_numbers.with_raw_response.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.external_connections.phone_numbers.with_streaming_response.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.phone_numbers.with_raw_response.update(
                phone_number_id="1234567889",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.external_connections.phone_numbers.with_raw_response.update(
                phone_number_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number = client.external_connections.phone_numbers.list(
            id="1293384261075731499",
        )
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.external_connections.phone_numbers.list(
            id="1293384261075731499",
            filter={
                "civic_address_id": {"eq": "19990261512338516954"},
                "location_id": {"eq": "19995665508264022121"},
                "phone_number": {
                    "contains": "+1970",
                    "eq": "+19705555098",
                },
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.external_connections.phone_numbers.with_raw_response.list(
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.external_connections.phone_numbers.with_streaming_response.list(
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.phone_numbers.with_raw_response.list(
                id="",
            )


class TestAsyncPhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.external_connections.phone_numbers.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.phone_numbers.with_raw_response.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.phone_numbers.with_streaming_response.retrieve(
            phone_number_id="1234567889",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.phone_numbers.with_raw_response.retrieve(
                phone_number_id="1234567889",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.external_connections.phone_numbers.with_raw_response.retrieve(
                phone_number_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.external_connections.phone_numbers.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.external_connections.phone_numbers.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.phone_numbers.with_raw_response.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.phone_numbers.with_streaming_response.update(
            phone_number_id="1234567889",
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.phone_numbers.with_raw_response.update(
                phone_number_id="1234567889",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.external_connections.phone_numbers.with_raw_response.update(
                phone_number_id="",
                id="1293384261075731499",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.external_connections.phone_numbers.list(
            id="1293384261075731499",
        )
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.external_connections.phone_numbers.list(
            id="1293384261075731499",
            filter={
                "civic_address_id": {"eq": "19990261512338516954"},
                "location_id": {"eq": "19995665508264022121"},
                "phone_number": {
                    "contains": "+1970",
                    "eq": "+19705555098",
                },
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.phone_numbers.with_raw_response.list(
            id="1293384261075731499",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.phone_numbers.with_streaming_response.list(
            id="1293384261075731499",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber], phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.phone_numbers.with_raw_response.list(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sim_card_order import SimCardOrder

__all__ = ["SimCardOrderRetrieveResponse"]


class SimCardOrderRetrieveResponse(BaseModel):
    data: Optional[SimCardOrder] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["WellKnownRetrieveProtectedResourceMetadataResponse"]


class WellKnownRetrieveProtectedResourceMetadataResponse(BaseModel):
    authorization_servers: Optional[List[str]] = None
    """List of authorization server URLs"""

    resource: Optional[str] = None
    """Protected resource URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AlphanumericSenderIDListParams"]


class AlphanumericSenderIDListParams(TypedDict, total=False):
    filter_messaging_profile_id: Annotated[str, PropertyInfo(alias="filter[messaging_profile_id]")]
    """Filter by messaging profile ID."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """Page number."""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """Page size."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AddressCreateParams"]


class AddressCreateParams(TypedDict, total=False):
    business_name: Required[str]
    """The business name associated with the address.

    An address must have either a first last name or a business name.
    """

    country_code: Required[str]
    """The two-character (ISO 3166-1 alpha-2) country code of the address."""

    first_name: Required[str]
    """The first name associated with the address.

    An address must have either a first last name or a business name.
    """

    last_name: Required[str]
    """The last name associated with the address.

    An address must have either a first last name or a business name.
    """

    locality: Required[str]
    """The locality of the address.

    For US addresses, this corresponds to the city of the address.
    """

    street_address: Required[str]
    """The primary street address information about the address."""

    address_book: bool
    """
    Indicates whether or not the address should be considered part of your list of
    addresses that appear for regular use.
    """

    administrative_area: str
    """The locality of the address.

    For US addresses, this corresponds to the state of the address.
    """

    borough: str
    """The borough of the address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    customer_reference: str
    """A customer reference string for customer look ups."""

    extended_address: str
    """
    Additional street address information about the address such as, but not limited
    to, unit number or apartment number.
    """

    neighborhood: str
    """The neighborhood of the address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    phone_number: str
    """The phone number associated with the address."""

    postal_code: str
    """The postal code of the address."""

    validate_address: bool
    """
    Indicates whether or not the address should be validated for emergency use upon
    creation or not. This should be left with the default value of `true` unless you
    have used the `/addresses/actions/validate` endpoint to validate the address
    separately prior to creation. If an address is not validated for emergency use
    upon creation and it is not valid, it will not be able to be used for emergency
    services.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from .comments import (
    CommentsResource,
    AsyncCommentsResource,
    CommentsResourceWithRawResponse,
    AsyncCommentsResourceWithRawResponse,
    CommentsResourceWithStreamingResponse,
    AsyncCommentsResourceWithStreamingResponse,
)
from .portouts import (
    PortoutsResource,
    AsyncPortoutsResource,
    PortoutsResourceWithRawResponse,
    AsyncPortoutsResourceWithRawResponse,
    PortoutsResourceWithStreamingResponse,
    AsyncPortoutsResourceWithStreamingResponse,
)
from .supporting_documents import (
    SupportingDocumentsResource,
    AsyncSupportingDocumentsResource,
    SupportingDocumentsResourceWithRawResponse,
    AsyncSupportingDocumentsResourceWithRawResponse,
    SupportingDocumentsResourceWithStreamingResponse,
    AsyncSupportingDocumentsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "ReportsResource",
    "AsyncReportsResource",
    "ReportsResourceWithRawResponse",
    "AsyncReportsResourceWithRawResponse",
    "ReportsResourceWithStreamingResponse",
    "AsyncReportsResourceWithStreamingResponse",
    "CommentsResource",
    "AsyncCommentsResource",
    "CommentsResourceWithRawResponse",
    "AsyncCommentsResourceWithRawResponse",
    "CommentsResourceWithStreamingResponse",
    "AsyncCommentsResourceWithStreamingResponse",
    "SupportingDocumentsResource",
    "AsyncSupportingDocumentsResource",
    "SupportingDocumentsResourceWithRawResponse",
    "AsyncSupportingDocumentsResourceWithRawResponse",
    "SupportingDocumentsResourceWithStreamingResponse",
    "AsyncSupportingDocumentsResourceWithStreamingResponse",
    "PortoutsResource",
    "AsyncPortoutsResource",
    "PortoutsResourceWithRawResponse",
    "AsyncPortoutsResourceWithRawResponse",
    "PortoutsResourceWithStreamingResponse",
    "AsyncPortoutsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.external_connections import upload_list_params, upload_create_params
from ...types.external_connections.upload import Upload
from ...types.external_connections.upload_retry_response import UploadRetryResponse
from ...types.external_connections.upload_create_response import UploadCreateResponse
from ...types.external_connections.upload_retrieve_response import UploadRetrieveResponse
from ...types.external_connections.upload_pending_count_response import UploadPendingCountResponse
from ...types.external_connections.upload_refresh_status_response import UploadRefreshStatusResponse

__all__ = ["UploadsResource", "AsyncUploadsResource"]


class UploadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UploadsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        number_ids: SequenceNotStr[str],
        additional_usages: List[Literal["calling_user_assignment", "first_party_app_assignment"]] | Omit = omit,
        civic_address_id: str | Omit = omit,
        location_id: str | Omit = omit,
        usage: Literal["calling_user_assignment", "first_party_app_assignment"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadCreateResponse:
        """
        Creates a new Upload request to Microsoft teams with the included phone numbers.
        Only one of civic_address_id or location_id must be provided, not both. The
        maximum allowed phone numbers for the numbers_ids array is 1000.

        Args:
          civic_address_id: Identifies the civic address to assign all phone numbers to.

          location_id: Identifies the location to assign all phone numbers to.

          usage: The use case of the upload request. NOTE: `calling_user_assignment` is not
              supported for toll free numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/external_connections/{id}/uploads",
            body=maybe_transform(
                {
                    "number_ids": number_ids,
                    "additional_usages": additional_usages,
                    "civic_address_id": civic_address_id,
                    "location_id": location_id,
                    "usage": usage,
                },
                upload_create_params.UploadCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadCreateResponse,
        )

    def retrieve(
        self,
        ticket_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRetrieveResponse:
        """
        Return the details of an Upload request and its phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ticket_id:
            raise ValueError(f"Expected a non-empty value for `ticket_id` but received {ticket_id!r}")
        return self._get(
            f"/external_connections/{id}/uploads/{ticket_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRetrieveResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: upload_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[Upload]:
        """
        Returns a list of your Upload requests for the given external connection.

        Args:
          filter: Filter parameter for uploads (deepObject style). Supports filtering by status,
              civic_address_id, location_id, and phone_number with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/external_connections/{id}/uploads",
            page=SyncDefaultFlatPagination[Upload],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    upload_list_params.UploadListParams,
                ),
            ),
            model=Upload,
        )

    def pending_count(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadPendingCountResponse:
        """
        Returns the count of all pending upload requests for the given external
        connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/external_connections/{id}/uploads/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadPendingCountResponse,
        )

    def refresh_status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRefreshStatusResponse:
        """
        Forces a recheck of the status of all pending Upload requests for the given
        external connection in the background.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/external_connections/{id}/uploads/refresh",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRefreshStatusResponse,
        )

    def retry(
        self,
        ticket_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRetryResponse:
        """
        If there were any errors during the upload process, this endpoint will retry the
        upload request. In some cases this will reattempt the existing upload request,
        in other cases it may create a new upload request. Please check the ticket_id in
        the response to determine if a new upload request was created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ticket_id:
            raise ValueError(f"Expected a non-empty value for `ticket_id` but received {ticket_id!r}")
        return self._post(
            f"/external_connections/{id}/uploads/{ticket_id}/retry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRetryResponse,
        )


class AsyncUploadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUploadsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        number_ids: SequenceNotStr[str],
        additional_usages: List[Literal["calling_user_assignment", "first_party_app_assignment"]] | Omit = omit,
        civic_address_id: str | Omit = omit,
        location_id: str | Omit = omit,
        usage: Literal["calling_user_assignment", "first_party_app_assignment"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadCreateResponse:
        """
        Creates a new Upload request to Microsoft teams with the included phone numbers.
        Only one of civic_address_id or location_id must be provided, not both. The
        maximum allowed phone numbers for the numbers_ids array is 1000.

        Args:
          civic_address_id: Identifies the civic address to assign all phone numbers to.

          location_id: Identifies the location to assign all phone numbers to.

          usage: The use case of the upload request. NOTE: `calling_user_assignment` is not
              supported for toll free numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/external_connections/{id}/uploads",
            body=await async_maybe_transform(
                {
                    "number_ids": number_ids,
                    "additional_usages": additional_usages,
                    "civic_address_id": civic_address_id,
                    "location_id": location_id,
                    "usage": usage,
                },
                upload_create_params.UploadCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadCreateResponse,
        )

    async def retrieve(
        self,
        ticket_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRetrieveResponse:
        """
        Return the details of an Upload request and its phone numbers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ticket_id:
            raise ValueError(f"Expected a non-empty value for `ticket_id` but received {ticket_id!r}")
        return await self._get(
            f"/external_connections/{id}/uploads/{ticket_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRetrieveResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: upload_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Upload, AsyncDefaultFlatPagination[Upload]]:
        """
        Returns a list of your Upload requests for the given external connection.

        Args:
          filter: Filter parameter for uploads (deepObject style). Supports filtering by status,
              civic_address_id, location_id, and phone_number with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/external_connections/{id}/uploads",
            page=AsyncDefaultFlatPagination[Upload],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    upload_list_params.UploadListParams,
                ),
            ),
            model=Upload,
        )

    async def pending_count(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadPendingCountResponse:
        """
        Returns the count of all pending upload requests for the given external
        connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/external_connections/{id}/uploads/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadPendingCountResponse,
        )

    async def refresh_status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRefreshStatusResponse:
        """
        Forces a recheck of the status of all pending Upload requests for the given
        external connection in the background.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/external_connections/{id}/uploads/refresh",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRefreshStatusResponse,
        )

    async def retry(
        self,
        ticket_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadRetryResponse:
        """
        If there were any errors during the upload process, this endpoint will retry the
        upload request. In some cases this will reattempt the existing upload request,
        in other cases it may create a new upload request. Please check the ticket_id in
        the response to determine if a new upload request was created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ticket_id:
            raise ValueError(f"Expected a non-empty value for `ticket_id` but received {ticket_id!r}")
        return await self._post(
            f"/external_connections/{id}/uploads/{ticket_id}/retry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UploadRetryResponse,
        )


class UploadsResourceWithRawResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create = to_raw_response_wrapper(
            uploads.create,
        )
        self.retrieve = to_raw_response_wrapper(
            uploads.retrieve,
        )
        self.list = to_raw_response_wrapper(
            uploads.list,
        )
        self.pending_count = to_raw_response_wrapper(
            uploads.pending_count,
        )
        self.refresh_status = to_raw_response_wrapper(
            uploads.refresh_status,
        )
        self.retry = to_raw_response_wrapper(
            uploads.retry,
        )


class AsyncUploadsResourceWithRawResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create = async_to_raw_response_wrapper(
            uploads.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            uploads.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            uploads.list,
        )
        self.pending_count = async_to_raw_response_wrapper(
            uploads.pending_count,
        )
        self.refresh_status = async_to_raw_response_wrapper(
            uploads.refresh_status,
        )
        self.retry = async_to_raw_response_wrapper(
            uploads.retry,
        )


class UploadsResourceWithStreamingResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create = to_streamed_response_wrapper(
            uploads.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            uploads.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            uploads.list,
        )
        self.pending_count = to_streamed_response_wrapper(
            uploads.pending_count,
        )
        self.refresh_status = to_streamed_response_wrapper(
            uploads.refresh_status,
        )
        self.retry = to_streamed_response_wrapper(
            uploads.retry,
        )


class AsyncUploadsResourceWithStreamingResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create = async_to_streamed_response_wrapper(
            uploads.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            uploads.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            uploads.list,
        )
        self.pending_count = async_to_streamed_response_wrapper(
            uploads.pending_count,
        )
        self.refresh_status = async_to_streamed_response_wrapper(
            uploads.refresh_status,
        )
        self.retry = async_to_streamed_response_wrapper(
            uploads.retry,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    NetworkListResponse,
    NetworkCreateResponse,
    NetworkDeleteResponse,
    NetworkUpdateResponse,
    NetworkRetrieveResponse,
    NetworkListInterfacesResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNetworks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        network = client.networks.create(
            name="test network",
        )
        assert_matches_type(NetworkCreateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.create(
            name="test network",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(NetworkCreateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.create(
            name="test network",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(NetworkCreateResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        network = client.networks.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.networks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        network = client.networks.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        )
        assert_matches_type(NetworkUpdateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(NetworkUpdateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(NetworkUpdateResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network_id` but received ''"):
            client.networks.with_raw_response.update(
                network_id="",
                name="test network",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        network = client.networks.list()
        assert_matches_type(SyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        network = client.networks.list(
            filter={"name": "test network"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        network = client.networks.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(NetworkDeleteResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(NetworkDeleteResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(NetworkDeleteResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.networks.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_interfaces(self, client: Telnyx) -> None:
        network = client.networks.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_interfaces_with_all_params(self, client: Telnyx) -> None:
        network = client.networks.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            filter={
                "name": "test interface",
                "type": "wireguard_interface",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_interfaces(self, client: Telnyx) -> None:
        response = client.networks.with_raw_response.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_interfaces(self, client: Telnyx) -> None:
        with client.networks.with_streaming_response.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_interfaces(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.networks.with_raw_response.list_interfaces(
                id="",
            )


class TestAsyncNetworks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.create(
            name="test network",
        )
        assert_matches_type(NetworkCreateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.create(
            name="test network",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(NetworkCreateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.create(
            name="test network",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(NetworkCreateResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(NetworkRetrieveResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.networks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        )
        assert_matches_type(NetworkUpdateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(NetworkUpdateResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.update(
            network_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            name="test network",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(NetworkUpdateResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network_id` but received ''"):
            await async_client.networks.with_raw_response.update(
                network_id="",
                name="test network",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.list()
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.list(
            filter={"name": "test network"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[NetworkListResponse], network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(NetworkDeleteResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(NetworkDeleteResponse, network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(NetworkDeleteResponse, network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.networks.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_interfaces(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_interfaces_with_all_params(self, async_client: AsyncTelnyx) -> None:
        network = await async_client.networks.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            filter={
                "name": "test interface",
                "type": "wireguard_interface",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_interfaces(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.with_raw_response.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        network = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_interfaces(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.with_streaming_response.list_interfaces(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            network = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[NetworkListInterfacesResponse], network, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_interfaces(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.networks.with_raw_response.list_interfaces(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .sim_card_groups import (
    SimCardGroupsResource,
    AsyncSimCardGroupsResource,
    SimCardGroupsResourceWithRawResponse,
    AsyncSimCardGroupsResourceWithRawResponse,
    SimCardGroupsResourceWithStreamingResponse,
    AsyncSimCardGroupsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "SimCardGroupsResource",
    "AsyncSimCardGroupsResource",
    "SimCardGroupsResourceWithRawResponse",
    "AsyncSimCardGroupsResourceWithRawResponse",
    "SimCardGroupsResourceWithStreamingResponse",
    "AsyncSimCardGroupsResourceWithStreamingResponse",
]
